package com.htek.mydialer;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;

import android.Manifest;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.RemoteException;
import android.telecom.Call;
import android.text.TextUtils;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.htek.voip.call.utils.CallMaker;
import com.htek.voip.cfg.parcel.VoipAccount;
import com.htek.voip.call.VoipInCallPresenter;
import com.htek.voip.cfg.parcel.VoipProfile;
import com.htek.voip.cfg.utils.CfgUtils;

import com.ipphone.bth.BthServiceAgent;
import ipphone.bth.ShadowBthDevice;
import ipphone.scene.BaseScene;
import ipphone.scene.SceneManager;
import ipphone.voip.VoipKeyEvent;

public class MainActivity extends AppCompatActivity {
    private final static String TAG = "MyDialer";
    private final Handler mHandler = new Handler(Looper.getMainLooper());
    private TextView mTvStatus;
    private final static int REQUEST_READ_PHONE_STATE = 1;

    private Call mCurCall = null;
    private Button mBtnSend, mBtnEnd, mBtnAnswer;

    private SceneManager mSceneManager;

    private BthServiceAgent mBthServiceAgent = null;

    public static final String ACTION_VOIP_ACCOUNT_STATUS_CHANGE = "com.htek.voip.service.action.account_status_change";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.READ_PHONE_STATE) != PackageManager.PERMISSION_GRANTED
            || ActivityCompat.checkSelfPermission(this, Manifest.permission.CALL_PHONE) != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.READ_PHONE_STATE, Manifest.permission.CALL_PHONE}, REQUEST_READ_PHONE_STATE);
            return;
        }
        init();
    }

    @Override
    protected void onResume() {
        //TODO, if you want to do something with btp50, you register it here and unregister it on onPause()
        mBthServiceAgent.register();

        //TODO, if you registerScene with priority bigger than SENCE_PRIORITY_HIGH but want to use htek
        // call interface, you need to registerScene here to handle some key events and unregisterScene
        // on onPause(). Or else, you can just registerScene once with priority lower than SENCE_PRIORITY_HIGH
        mSceneManager.registerScene(SceneManager.SENCE_PRIORITY_MAX, mSceneClient);
        super.onResume();
    }

    @Override
    protected void onPause() {
        mBthServiceAgent.unregister();
        mSceneManager.unregisterScene(mSceneClient);
        super.onPause();
    }

    private void init(){
        com.htek.voip.call.Base.init(this);

        EditText editText = findViewById(R.id.tvNum);
        mBtnSend = findViewById(R.id.btnSend);
        mBtnSend.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                /*Context context = v.getContext();
                @CallMaker.CallResult int ret = new CallMaker().account(CfgUtils.getDefaultAccount(context))
                        .asNormalCall().number(editText.getText().toString()).video(false).autoUseDialPlan().call(context);
                if (ret != CallMaker.CALL_RESULT_OK) {
                    Toast.makeText(context, CallMaker.strResult(context, ret), Toast.LENGTH_SHORT).show();
                }*/
                Context context = v.getContext();
                dialNum(context, CfgUtils.getDefaultAccount(context), editText.getText().toString());
            }
        });

        mBtnEnd = findViewById(R.id.btnEnd);
        mBtnEnd.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if(mCurCall == null){
                    return;
                }

                switch(mCurCall.getState()){
                    default:
                        mCurCall.disconnect();
                        break;
                    case Call.STATE_RINGING:
                        mCurCall.reject(false, "");
                        break;
                }
            }
        });

        mBtnAnswer = findViewById(R.id.btnAnswer);
        mBtnAnswer.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if(mCurCall == null){
                    return;
                }

                if(mCurCall.getState() != Call.STATE_RINGING){
                    return;
                }

                mCurCall.answer(0);
            }
        });

        /*Intent intent = getIntent();
        if (intent != null && Intent.ACTION_DIAL.equals(intent.getAction())) {
            String num = intent.getDataString();
            Log.i(TAG, "on intent dial: "+num);
        }*/

        VoipAccount account = CfgUtils.getDefaultAccount(this);
        if(account != null){
            Log.i(TAG, "defaultAccount: "+account+" pw: "+account.getAuthPassword());
        }

        VoipProfile profile = CfgUtils.getProfile(this, 0);
        Log.i(TAG, "profile 0: "+profile);

        mTvStatus = findViewById(R.id.tvStatus);

        VoipInCallPresenter voipInCallPresenter = VoipInCallPresenter.getInstance();
        voipInCallPresenter.registerPresenterListener(new VoipInCallPresenter.PresenterListener() {
            @Override
            public void onCallAdded(Call call) {
                //VoipInCallPresenter.PresenterListener.super.onCallAdded(call);
                Log.i(TAG, "voipInCallPresenter onCallAdded: "+call);
                switch(call.getState()){
                    case Call.STATE_RINGING:
                        mCurCall = call;
                        mBtnSend.setVisibility(View.GONE);
                        mBtnAnswer.setVisibility(View.VISIBLE);
                        call.registerCallback(mCallCallback, mHandler);
                        mTvStatus.setText("Incoming...");
                        break;
                    case Call.STATE_CONNECTING:
                        mCurCall = call;
                        mBtnSend.setVisibility(View.GONE);
                        mBtnAnswer.setVisibility(View.GONE);
                        call.registerCallback(mCallCallback, mHandler);
                        mTvStatus.setText("Connecting...");
                        break;
                }
            }
        });

        mSceneManager = getSystemService(SceneManager.class);

        mBthServiceAgent = new BthServiceAgent(this);

        registerReceiver(mAccountReceiver, new IntentFilter(ACTION_VOIP_ACCOUNT_STATUS_CHANGE));
    }

    Call.Callback mCallCallback = new Call.Callback() {
        @Override
        public void onStateChanged(Call call, int state) {
            VoipInCallPresenter.getInstance().updateVoipAudioStream(state);
            switch (state){
                case Call.STATE_ACTIVE:
                    VoipInCallPresenter.getInstance().useHandfree();
                    mBtnSend.setVisibility(View.GONE);
                    mBtnAnswer.setVisibility(View.GONE);
                    mTvStatus.setText("Talking...");
                    break;
                default:
                    break;
            }
            super.onStateChanged(call, state);
        }

        @Override
        public void onCallDestroyed(Call call) {
            mTvStatus.setText("No Call");
            mBtnSend.setVisibility(View.VISIBLE);
            mBtnAnswer.setVisibility(View.GONE);
            mCurCall = null;
            super.onCallDestroyed(call);
        }
    };

    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        if (intent != null && Intent.ACTION_DIAL.equals(intent.getAction())) {
            String num = intent.getDataString();
            Log.i(TAG, "onNewIntent dial: "+num);
        }
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
//        switch(keyCode){
//            case 302:
//            case 303:
//                Log.i(TAG, "onKeyDown: "+keyCode);
//                return true;
//        }
        return super.onKeyDown(keyCode, event);
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if(requestCode == REQUEST_READ_PHONE_STATE){
            if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                init();
            }else{
                Toast.makeText(this, "Read phone state permission must be granted", Toast.LENGTH_SHORT).show();
                finish();
            }
        }
    }

    /**
     * launch htek dialpad activity
     * @param accountIndex - different account. Here 0 is an example. it can be 0, 1, ..., depends on your config
     * @param number - the number you want to transport to dialpad edit view. Leave empty or null
     *               to trigger empty dial.
     */
    private void launchHtekDialpad(Context context, int accountIndex, String number){
        VoipAccount account = CfgUtils.getAccount(context, accountIndex);
        if(account != null){
            CfgUtils.setDefaultAccount(context, account);
        }
        Intent i = new Intent(Intent.ACTION_DIAL);
        if(!TextUtils.isEmpty(number)){
            i.setData(Uri.parse("tel:" + number));
        }
        context.startActivity(i);
    }

    /**
     * dial number directly. Htek calling activity will show if MyDialer has not been set as "Phone app",
     * else, htek calling activity will NOT show.
     * @param accountIndex - different account. Here 0 is an example. it can be 0, 1, ..., depends on your config
     * @param number - the number you want to dial.
     */
    private void dialNum(Context context, int accountIndex, String number){
        dialNum(context, CfgUtils.getAccount(context, accountIndex), number);
    }

    private void dialNum(Context context, VoipAccount account, String number){
        if(TextUtils.isEmpty(number) || account == null){
            return;
        }
        @CallMaker.CallResult int ret = new CallMaker().account(account)
                .asNormalCall().number(number).video(false).autoUseDialPlan().call(context);
        if (ret != CallMaker.CALL_RESULT_OK) {
            Toast.makeText(context, CallMaker.strResult(context, ret), Toast.LENGTH_SHORT).show();
        }
    }

    private BaseScene mSceneClient = new BaseScene() {

        /*//can omit empty functions now
        @Override
        public void enter() throws RemoteException {

        }

        @Override
        public void exit() throws RemoteException {

        }

        @Override
        public void onDialpadPressIfDreamingOrNoninteractive(char c, boolean replace) throws RemoteException {

        }

        @Override
        public boolean onDialpadPress(char c, boolean replace) throws RemoteException {
            return false;
        }*/

        @Override
        public boolean onUnhandleKeyEvent(KeyEvent event) throws RemoteException {
            if (event.getAction() == KeyEvent.ACTION_DOWN && event.getRepeatCount() == 0) {
                switch (event.getKeyCode()) {
                    case VoipKeyEvent.KEYCODE_VOIP_REDIAL:
                        showToast("redial click");
                        break;
                    case VoipKeyEvent.KEYCODE_VOIP_FORWARD:
                        showToast("forward click");
                        break;
                    case VoipKeyEvent.KEYCODE_VOIP_HOLD:
                        showToast("hold click");
                        break;
                    case VoipKeyEvent.KEYCODE_VOIP_SPEAKER:
                        showToast("speaker click");
                        mSceneManager.openHandfree();
                        launchHtekDialpad(MainActivity.this,0, null);
                        //launchHtekDialpad(MainActivity.this,0, "0009");
                        break;
                    case VoipKeyEvent.KEYCODE_VOIP_HEADSET:
                        showToast("headset click");
                        mSceneManager.openHeadset();
                        launchHtekDialpad(MainActivity.this,0, null);
                        break;
                    case VoipKeyEvent.KEYCODE_VOIP_MUTE:
                        showToast("mute click");
                        break;
                    case VoipKeyEvent.KEYCODE_VOIP_OFFHOOK:
                        showToast("offhook");
                        mSceneManager.openHandset();
                        launchHtekDialpad(MainActivity.this, 0, null);
                        //launchHtekDialpad(MainActivity.this,1, "1009");
                        break;
                    case VoipKeyEvent.KEYCODE_VOIP_ONHOOK:
                        showToast("onHook");
                        /*if (mSceneManager.isHandsetOn()) {
                            mSceneManager.openHandfree();
                        }*/
                        break;
                    case VoipKeyEvent.KEYCODE_VOIP_VIDEO:
                        showToast("video click");
                        break;
                    case VoipKeyEvent.KEYCODE_VOIP_USB_KEY:
                        showToast("usb click");
                        break;
                    case VoipKeyEvent.KEYCODE_VOIP_BLUETOOTH_KEY:
                        showToast("bluetooth click");
                        break;
                    case VoipKeyEvent.KEYCODE_VOIP_WIFI_KEY:
                        showToast("wifi click");
                        break;
                    case VoipKeyEvent.KEYCODE_USBHEADSET_OFFHOOK:
                        showToast("usb headset offhook");
                        break;
                    case VoipKeyEvent.KEYCODE_USBHEADSET_ONHOOK:
                        showToast("usb headset onhook");
                        break;
                    case VoipKeyEvent.KEYCODE_USBHEADSET_REDIAL:
                        showToast("usb headset redial");
                        break;
                }
            }
            return true;
        }

        /*@Override
        public boolean onAudioDeviceChanged() throws RemoteException {
            return false;
        }

        @Override
        public boolean onEhsEvent(int ehsEvent) throws RemoteException {
            return false;
        }*/

        @Override
        public String getName() throws RemoteException {
            return "MyDialer";
        }

        /*@Override
        public void offHook(int keyCode) throws RemoteException {

        }*/

        @Override
        public boolean onBthClickCmd(ShadowBthDevice dev, int cmd) throws RemoteException {
            switch(cmd){
                case BthServiceAgent.EVENT_CMD_EAR_CLCK:
                    if(dev.getMainDev() != ShadowBthDevice.Dev.EAR){
                        mBthServiceAgent.sendCmd(BthServiceAgent.CMD_MDEV_EAR);
                    } else {
                        mBtnEnd.post(()->mBtnEnd.performClick());
                    }
                    break;
                case BthServiceAgent.EVENT_CMD_SPK_CLCK:
                    if(dev.getMainDev() != ShadowBthDevice.Dev.SPK){
                        mBthServiceAgent.sendCmd(BthServiceAgent.CMD_MDEV_SPK);
                    }  else {
                        mBtnEnd.post(()->mBtnEnd.performClick());
                    }
                    break;
                case BthServiceAgent.EVENT_CMD_EAR_DCLCK:
                case BthServiceAgent.EVENT_CMD_SPK_DCLCK:
                    mBthServiceAgent.redial(cmd);
                    break;
            }
            return true;
        }

        /*@Override
        public boolean onBthRawCmd(ShadowBthDevice dev, String cmd) throws RemoteException {
            return false;
        }

        @Override
        public boolean onFunctionKeyEvent(KeyEvent event) throws RemoteException {
            return false;
        }*/

        private void showToast(String msg) {
            MainActivity.this.runOnUiThread(() -> Toast.makeText(MainActivity.this, msg, Toast.LENGTH_SHORT).show());
        }
    };

    @Override
    protected void onDestroy() {
        super.onDestroy();
        unregisterReceiver(mAccountReceiver);
    }

    private BroadcastReceiver mAccountReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            VoipAccount account = CfgUtils.getDefaultAccount(MainActivity.this);
            if(account != null){
                Toast.makeText(MainActivity.this, "account active", Toast.LENGTH_SHORT).show();
            }
        }
    };

}